package com.flyfish.framework.query.adaptor;

import org.bson.BsonRegularExpression;
import org.springframework.data.domain.Example;
import org.springframework.data.geo.Circle;
import org.springframework.data.geo.Point;
import org.springframework.data.geo.Shape;
import org.springframework.data.mongodb.core.geo.GeoJson;
import org.springframework.data.mongodb.core.schema.JsonSchemaObject;

import java.util.Collection;
import java.util.regex.Pattern;

/**
 * 查询适配器
 *
 * @author wangyu
 * 由工厂进行实例化，并输出
 */
public interface CriteriaAdaptor {

    CriteriaAdaptor and(String key);

    CriteriaAdaptor is(Object value);

    CriteriaAdaptor isNull();

    CriteriaAdaptor isNullValue();

    CriteriaAdaptor ne(Object value);

    CriteriaAdaptor lt(Object value);

    CriteriaAdaptor lte(Object value);

    CriteriaAdaptor gt(Object value);

    CriteriaAdaptor gte(Object value);

    CriteriaAdaptor in(Object... values);

    CriteriaAdaptor in(Collection<?> values);

    CriteriaAdaptor nin(Object... values);

    CriteriaAdaptor nin(Collection<?> values);

    CriteriaAdaptor mod(Number value, Number remainder);

    CriteriaAdaptor all(Object... values);

    CriteriaAdaptor all(Collection<?> values);

    CriteriaAdaptor size(int size);

    CriteriaAdaptor exists(boolean value);

    CriteriaAdaptor sampleRate(double sampleRate);

    CriteriaAdaptor type(int typeNumber);

    CriteriaAdaptor type(JsonSchemaObject.Type... types);

    CriteriaAdaptor type(Collection<JsonSchemaObject.Type> types);

    CriteriaAdaptor not();

    CriteriaAdaptor regex(String regex);

    CriteriaAdaptor regex(String regex, String options);

    CriteriaAdaptor regex(Pattern pattern);

    CriteriaAdaptor regex(BsonRegularExpression regex);

    CriteriaAdaptor withinSphere(Circle circle);

    CriteriaAdaptor within(Shape shape);

    CriteriaAdaptor near(Point point);

    CriteriaAdaptor nearSphere(Point point);

    CriteriaAdaptor intersects(GeoJson geoJson);

    CriteriaAdaptor maxDistance(double maxDistance);

    CriteriaAdaptor minDistance(double minDistance);

    CriteriaAdaptor elemMatch(CriteriaAdaptor criteria);

    CriteriaAdaptor alike(Example<?> sample);

    CriteriaAdaptor orOperator(CriteriaAdaptor... criteria);

    CriteriaAdaptor orOperator(Collection<CriteriaAdaptor> criteria);

    CriteriaAdaptor norOperator(CriteriaAdaptor... criteria);

    CriteriaAdaptor norOperator(Collection<CriteriaAdaptor> criteria);

    CriteriaAdaptor andOperator(CriteriaAdaptor... criteria);

    CriteriaAdaptor andOperator(Collection<CriteriaAdaptor> criteria);
}
